import copy

#import peak.events
#from peak.events import trellis
import mext.reaction.collections as trellis

import currypy

import pypatterns.filter as FilterModule

class Table(object):
    
    JOIN_CROSS = 'cross join'
    JOIN_INNER = 'inner join'
    JOIN_LEFT = 'left join'
    JOIN_RIGHT = 'right join'
    JOIN_FULL = 'full join'

    NULL = object()

    COLUMN_ATTRIBUTE_UNIQUE = 'unique'

    
    @staticmethod
    def reduceRetrieve(table, filter, columns, initialValue=None):
        if initialValue is None:
            initialValue = []
        return reduce(lambda x, y: x+y,
                      table.retrieve(filter, columns),
                      initialValue)
    
    
    def __init__(self, name=None):
        self.name(name)
        self.rows(trellis.List([]))
        
        # columns defaults to a set
        # where the keys are the names of the columns
        # and the values are the attributes
        self.columns(trellis.Dict(**{}))
        
        return

    def __getstate__(self):
        odict = self.__dict__.copy()
        odict['_rows'] = list(odict['_rows'])
        odict['_columns'] = dict(
            (key, set(value)) 
            for key, value in odict['_columns'].iteritems())
        return odict

    def __setstate__(self, odict):
        self.rows(trellis.List(odict['_rows']))
        d = dict((key, trellis.Set(value))
                 for key, value in odict['_columns'].iteritems())
        self.columns(trellis.Dict(**d))
        return
    
    def __eq__(self, other):
        for difference in self.getDifferencesWith(other):
            return False
        return True

    def __deepcopy__(self, memo):
        if self in memo:
            return memo[self]
        
        result = createTable(self.name(),
                             self.columns())
        for row in self.rows():
            newRow = result.addRow()
            for column in self.columns():
                columnValue = row.getColumn(column)
                columnValueCopy = copy.deepcopy(columnValue, memo)
                newRow.setColumn(column, columnValueCopy)
                
        memo[self] = result
        
        return result
    
    
    def getDifferencesWith(self, other):
        if not type(self) is type(other):
            yield 'not the same type'
        if not self.name() == other.name():
            yield 'self is named %s, but other is named %s' % (self.name(), other.name())
        if not set(self.columns()) == set(other.columns()):
            yield 'self has columns %s, but other has columns %s' % (self.columns(), other.columns())
        if not len(self.rows()) == len(other.rows()):
            yield 'self has %s rows, other has %s rows' % (len(self.rows()), len(other.rows()))
            
        # to optimize, lets sort both 
        columnComparatorMap = dict([(x, cmp) for x in self.columns()])
        rowComparator = currypy.Curry(Row.comparator,
                                          columnComparatorMap=columnComparatorMap)
        leftRows = sorted(self.rows(), rowComparator)
        rightRows = sorted(other.rows(), rowComparator)
        for leftRow, rightRow in zip(leftRows, rightRows):
            if rowComparator(leftRow, rightRow) is not 0:
                yield 'self has row %s, other has row %s' % (leftRow._values, rightRow._values)
                break
            pass
        
        raise StopIteration
    

    def name(self, value=None):
        if value is not None:
            self._name = value
        if not hasattr(self, '_name'):
            self._name = None
        return self._name

    def rows(self, value=None):
        if value is not None:
            self._rows = value
        if not hasattr(self, '_rows'):
            self._rows = None
        return self._rows

    def columns(self, value=None):
        if value is not None:
            self._columns = value
        if not hasattr(self, '_columns'):
            self._columns = None
        return self._columns

    
    def rowCount(self):
        return len(self.rows())
    
    def hasColumn(self, column):
        return column in self.columns()
    
    def addColumn(self, column):
        self.columns()[column] = trellis.Set([])
        return
    
    def addColumnAttribute(self, column, attribute):
        if not self.hasColumn(column):
            raise KeyError('table does not have column %s' % column)
        
        self.columns()[column].add(attribute)
        return
    
    
    def addRow(self, row=None):
        if row is None:
            row = Row(self)
        self.rows().append(row)
        return row


    def clear(self):
        filter = FilterModule.TRUE_FILTER
        return self.removeRows(filter)

    
    def removeRows(self, filter):
        rowsToRemove = [x for x in filter.wrap(self.rows())]
        map(self.rows().remove, rowsToRemove)
        return rowsToRemove

    
    def retrieve(self, filter=None, columns=None):
        if filter is None:
            filter = FilterModule.TRUE_FILTER
            
        if columns is None:
            columns = self.columns()
        
        for row in filter.wrap(self.rows()):
            rowValues = [row.getColumn(column) for column in columns]
            yield rowValues
            
        raise StopIteration

    
    def retrieveForModification(self, filter):
        for row in filter.wrap(self.rows()):
            yield row
            
        raise StopIteration
    
    
    def hasRow(self, filter):
        for row in filter.wrap(self.rows()):
            return True
        return False

    def update(self, other):
        if not set(self.columns.keys()).issuperset(other.columns().keys()):
            raise NotImplementedError
        otherColumns = other.columns().keys()
        for rowValues in other.retrieve(FilterModule.TRUE_FILTER, otherColumns):
            row = self.addRow()
            map(row.setValue, otherColumns, rowValues)
            pass
        return

    def addObjectAsRow(self, objectToAdd, objectKeyFunctionsMap):
        row = self.addRow()
        for column in self.columns():
            objectKeyFunction = objectKeyFunctionsMap[column]
            row.setColumn(column, objectKeyFunction(objectToAdd))
            pass
        return
    
    
    # END class Table
    pass

class Row(object):

    @staticmethod
    def comparator(left, right, columnComparatorMap):
        for column, comparator in columnComparatorMap.iteritems():
            compareValue = comparator(left.getColumn(column),
                                      right.getColumn(column))
            if not compareValue is 0:
                return compareValue
            pass
        return 0
    
    def __init__(self, table=None):
        
        self.table(table)

        self.values(trellis.Dict(**{}))
        
        return

    def __getstate__(self):
        odict = self.__dict__.copy()
        odict['_values'] = dict(odict['_values'])
        return odict
    
    def __setstate__(self, odict):
        self.table(odict['_table'])
        d = odict['_values']
        values = trellis.Dict(**d)
        self.values(values)
        return    
    
    def values(self, value=None):
        if value is not None:
            self._values = value
        if not hasattr(self, '_values'):
            self._values = None
        return self._values

    def table(self, value=None):
        if value is not None:
            self._table = value
        if not hasattr(self, '_table'):
            self._table = None
        return self._table

    def hasColumn(self, column):
        return self.table().hasColumn(column)
    
    def setColumn(self, column, value):
        if not self.hasColumn(column):
            raise KeyError('row does not have column %s' % column)

        self.values()[column] = value
        return

    
    def getColumn(self, column):
        return self.values().get(column, Table.NULL)
    
    # END class Row
    pass

class ColumnValueFilter(FilterModule.ObjectFilter):
    
    """
    This is a filter on a row,
    and returns true if the value of the column in the row matches
    """
    
    def __init__(self, column, valueFilter):
        FilterModule.ObjectFilter.__init__(self,
            valueFunction = lambda: valueFilter,
            objectFunction = lambda x: x.getColumn(column),
            compareFunction = lambda x, y: y.matches(x)
        )
        self._column = column
        self._valueFilter = valueFilter
        
        return
    
    # END ColumnValueFilter
    pass


class ColumnJoinFilter(FilterModule.ObjectFilter):
    
    def __init__(self, left, right):
        FilterModule.ObjectFilter.__init__(
            self,
            valueFunction = lambda: True,
            objectFunction = lambda x: x[0].getColumn(self._left) == x[1].getColumn(self._right)
        )
        self._left = left
        self._right = right
        return
    
    # END JoinFilter
    pass


class JoinFilter(FilterModule.CompositeFilter):
    """
    This class defines the default/basic join filter
    that is used as the industry standard
    """
    
    def __init__(self, columnMap):
        FilterModule.CompositeFilter.__init__(
            self, FilterModule.AndAccumulator())
        
        map(self.addFilter, [ColumnJoinFilter(x, y) for x,y in columnMap.iteritems()])
        
        return
    
    # END class JoinFilter
    pass


def createTable(name, columns):
    table = Table(name=name)
    map(table.addColumn, columns)

    return table


def join(joinInfo):
    # left, right, aFilter, anOperator, joinType=None):
    """
    @param aFilter determines whether the rows should be joined
    @param anOperator does the actual joining
    """

    left = joinInfo['left']
    right = joinInfo['right']
    theFilter = joinInfo['filter']
    theOperator = joinInfo['operator']
    leftColumns = joinInfo['left columns']
    rightColumns = joinInfo['right columns']
    name = joinInfo['name']
    joinType = joinInfo['join type']
    
    
    # currently implementing in a code clean fashion
    # but should attempt to optimize instead
    # leftRows and otherRows will keep track of the unmatched rows
    leftRows = set(left.rows())
    rightRows = set(right.rows())
    joinedTable = createTable(name, leftColumns.values()+rightColumns.values())
    for leftRow in left.rows():
        for rightRow in right.rows():
            if not theFilter.matches((leftRow, rightRow)):
                continue
            
            leftRows.discard(leftRow)
            rightRows.discard(rightRow)
            
            theOperator(joinedTable, leftRow, rightRow)
            pass
        pass
    
    if joinType in [Table.JOIN_FULL, Table.JOIN_LEFT]:
        for row in leftRows:
            theOperator(joinedTable, leftRow, None)
        pass
    
    if joinType in [Table.JOIN_FULL, Table.JOIN_RIGHT]:
        for row in rightRows:
            theOperator(joinedTable, None, rightRow)
        pass
            
    return joinedTable


def defaultJoinOperator(table, left, right, leftColumnMap, rightColumnMap):
    row = table.addRow()
    
    # if not left, then we are doing a right join
    for sourceColumn, targetColumn in leftColumnMap.iteritems():
        valueToSet = Table.NULL
        if left:
            valueToSet = left.getColumn(sourceColumn)
        row.setColumn(targetColumn, valueToSet)
        pass

    for sourceColumn, targetColumn in rightColumnMap.iteritems():
        valueToSet = Table.NULL
        if right:
            valueToSet = right.getColumn(sourceColumn)
        row.setColumn(targetColumn, valueToSet)
        pass
    return
    



import logging
import sys
import traceback

class Validator(object):

    def validate(self):
        raise NotImplementedError('have not implemented validate')

    # END class Validator
    pass


class NotValidator(Validator):

    def __init__(self, validator):
        self._validator = validator
        return

    def validate(self):
        return not self._validator.validate()

    # END class NotValidator
    pass

VALIDATOR_SUCCEED = Validator()
VALIDATOR_SUCCEED.validate = lambda: True

VALIDATOR_FAIL = NotValidator(VALIDATOR_SUCCEED)


class ValidationError(Exception):
    pass

class ExecutionError(Exception):
    pass

class Command(object):

    def __init__(self):
        self._hasExecuted = False
        self.validator(VALIDATOR_SUCCEED)
        return

    def validator(self, value=None):
        if value is not None:
            self._validator = value
        return self._validator

    def do(self):
        """
        This method is public API for executing commands
        and the one that's called by external objects,
        e.g. the command manager or the unittest
        The method in turn calls self.execute()
        """
        if self._hasExecuted:
            raise ValidationError('%s has already executed' % self)
        if not self._validator.validate():
            raise ValidationError('%s failed validation' % self)

        try:
            self.execute()
            self._hasExecuted = True
        except Exception, e:
            self.handleExecuteError()
            raise
        return

    def handleExecuteError(self):
        return

    def execute(self):
        """
        subclass just need to implement this
        and the complimentary unexecute method
        """
        raise NotImplementedError('have not implemented execute')

    def undo(self):
        """
        This method is public API for undoing commands
        and the one that's called by external objects,
        e.g. the command manager or the unittest
        The method in turn calls self.unexecute()
        """
        if not self._hasExecuted:
            raise ValidationError('%s has not yet executed' % self)

        self.unexecute()
        self._hasExecuted = False
        return

    def unexecute(self):
        """
        subclass just need to implement this
        and the complimentary execute method
        """
        raise NotImplementedError('have not implemented unexecute')

    # END class Command
    pass 


class CompositeCommand(Command):

    def __init__(self):
        Command.__init__(self)

        # we need both the list and the map
        # even though the keys to the map are the elements of the list
        # because the list maintains execution order
        # and the map maintains the command builders
        self._subcommands = []
        self._commandBuilders = {}

        self._executedCommands = []
        return


    def addCommand(self, command):
        self._subcommands.append(command)
        self._commandBuilders[command] = []
        return


    def addCommandBuilder(self, command, commandBuilder):
        """
        command builders are associated with a particular command
        they are called in the order in which they are appended
        """
        self._commandBuilders[command].append(commandBuilder)
        return

    def handleExecuteError(self):
        self.unexecute()
        return

    def execute(self):
        # TODO:
        # also need command builders that apply to the whole sequence
        # and not tied to any specific subcommand

        map(self._executeSubcommand, self._subcommands)

        pass


    def _executeSubcommand(self, command):
        generatedCommands = [command]
        for commandBuilder in self._commandBuilders[command]:
            commandBuilder.addCommandsPreExecute(generatedCommands)
            pass

        map(self._executeRawCommand, generatedCommands)

        # now run the command builders
        # and see if any commands need to be generated 
        # and executed post execution of the original command
        generatedCommands = []
        for commandBuilder in self._commandBuilders[command]:
            commandBuilder.addCommandsPostExecute(generatedCommands)
            pass

        map(self._executeRawCommand, generatedCommands)
        return


    def _executeRawCommand(self, command):
        try:
            command.do()
            self._executedCommands.append(command)
        except Exception, e:
            # undo the command
            # and raise
            # the thing executing self
            # is responsible for calling the undo
            logging.debug("failed _executeRawCommand >> %s" % e)
            # self.unexecute()
            raise
        return


    def unexecute(self):
        for subcommand in self._executedCommands.__reversed__():
            # need to call undo
            # and not unexecute
            # because we need undo to revert
            # the command's internal state values
            subcommand.undo()
            pass
        return

    # END class CompositeCommand
    pass



class CommandManager(object):

    def __init__(self):

        self._commandHistory = []
        self._endIndex = 0

        return

    def stackTrace(self, value=None):
        if value is not None:
            self._stackTrace = value
        if not hasattr(self, '_stackTrace'):
            self._stackTrace = None
        return self._stackTrace

    def do(self, command):
        try:
            command.do()

            # since the command was successful
            # we want to remove all the commands 
            # that have been undone 
            # but could still be redone
            numCommands = len(self._commandHistory)
            if not self._endIndex is numCommands:
                self._commandHistory = self._commandHistory[:self._endIndex-1]

            self._commandHistory.append(command)
            self._endIndex = len(self._commandHistory)
        except Exception, e:

            exceptionType, exceptionValue, exceptionTraceback = sys.exc_info()
            stackTrace = traceback.format_exception(
                exceptionType, exceptionValue, exceptionTraceback)
            self.stackTrace(stackTrace)

            return False

        return True

    def undo(self):
        if self._endIndex == 0:
            raise ValueError('cannot undo on an empty history')
        commandToUndo = self._commandHistory[self._endIndex-1]
        commandToUndo.undo()
        self._endIndex = self._endIndex - 1
        return
        

    # END class CommandManager
    pass


class CommandBuilder(object):
    """
    command builders allow for commands to be generated 
    during the execution of a composite command
    such that the generated commands become subcommands
    of that composite command
    """

    def __init__(self, command):
        self._command = command
        return

    def addCommandsPreExecute(self, commands):
        return

    def addCommandsPostExecute(self, commands):
        return

    # END class CommandBuilder
    pass



import os
import sys
import unittest
import logging

APP_ROOT = os.getenv('APP_ROOT')
sys.path.insert(0, '%s/pypatterns/src' % APP_ROOT)
sys.path.insert(0, '%s/currypy/src' % APP_ROOT)


import pypatterns.command as CommandModule
import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule
import pypatterns.relational.commands as RelationalCommandModule

sys.path.insert(0,"../data")

class TestCase(unittest.TestCase):

    COLUMNS = ['column1', 'column2', 'column3']
    
    
    def testTable(self):
        columns = TestCase.COLUMNS

        l = []

        command = CommandModule.CompositeCommand()

        command.addCommand(
            RelationalCommandModule.CreateTableCommand(
                'test',
                columns,
                l.append,
                l.remove
                )
            )

        command.do()
        table = l[0]

        command = CommandModule.CompositeCommand()
        rowValuesList = [
            [1,2,3],
            [1,'2','3'],
            [None,None,[]]
        ]
        for rowValues in rowValuesList:
            #row = table.addRow()
            #map(row.setColumn, columns, rowValues)
            l = []
            addRowCommand = RelationalCommandModule.AddRowCommand(
                table,
                l.append
                )
            command.addCommand(addRowCommand)
            for column, value in zip(columns, rowValues):
                commandBuilder = RelationalCommandModule.SetColumnValueCommandBuilder(
                    addRowCommand,
                    column,
                    value)
                command.addCommandBuilder(addRowCommand, commandBuilder)
                pass
            pass
                

        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1',
            FilterModule.ObjectFilter(
                valueFunction=lambda: 1,
                objectFunction=lambda x: x)
            )
        
        for actualValues, expectedValues in \
            zip(table.retrieve(columnValueFilter, columns[1:]), rowValuesList):
            
            assert actualValues == expectedValues[1:]
            
            
        return


    def testRemoveRows(self):
        table = self.buildDefaultTable()

        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1', FilterModule.IdentityFilter(1))
        
        command = RelationalCommandModule.RemoveRowsCommand(
            table, columnValueFilter)

        count = len(table.rows())
        assert count is not 0

        command.do()
        assert len(table.rows()) is 0

        command.undo()
        assert len(table.rows()) is count
        return

    
    def testModifyRow(self):
        table = self.buildDefaultTable()

        originalValue = 1
        column = 'column1'

        columnValueFilter = RelationalModule.ColumnValueFilter(
            column, FilterModule.IdentityFilter(originalValue))

        rows = [x for x in table.retrieveForModification(columnValueFilter)]

        command = RelationalCommandModule.SetColumnValueUsingRowFilterCommand(
            columnValueFilter, table, column, 2)
        command.do()
            
        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1', FilterModule.IdentityFilter(2))
        for rowValues in table.retrieve(columnValueFilter, [column]):
            assert rowValues[0] is 2
        
        command.undo()
        
        for row in rows:
            assert row.getColumn(column) is originalValue

        return


    def buildDefaultTable(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test', columns)

        row = table.addRow()
        rowValues = [1, '1', None]
        map(row.setColumn, columns, rowValues)
        
        return table

    
    # END class TestCase
    pass


    

def main():
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import logging
import sys
import os
import unittest

APP_ROOT = os.getenv('APP_ROOT')
sys.path.insert(0, '%s/pypatterns/src' % APP_ROOT)

import pypatterns.command as CommandModule

class HasKeyValidator(CommandModule.Validator):

    def __init__(self, map, key):
        self._map = map
        self._key = key
        return

    def validate(self):
        return self._key in self._map

    # END HasKeyValidator
    pass


class AddKeyCommand(CommandModule.Command):

    def __init__(self, map, key):
        CommandModule.Command.__init__(self)
        self._map = map
        self._key = key
        return

    def execute(self):
        self._map[self._key] = None
        return
    
    def unexecute(self):
        del self._map[self._key]
        return

    # END class AddKeyCommand
    pass


class FailCommand(CommandModule.Command):

    def execute(self):
        raise CommandModule.ExecutionError('this command always fails')

    # END class FailCommand
    pass

class SetValueCommand(CommandModule.Command):
    
    def __init__(self, map, key, value):
        CommandModule.Command.__init__(self)
        self._map = map
        self._key = key
        self._value = value
        return

    def execute(self):
        self._originalValue = self._map[self._key]
        self._map[self._key] = self._value
        return

    def unexecute(self):
        self._map[self._key] = self._originalValue
        delattr(self, '_originalValue')
        return
        

    # END SetValueCommand
    pass


class DeleteKeyCommand(CommandModule.Command):

    def __init__(self, map, key):
        CommandModule.Command.__init__(self)
        self._map = map
        self._key = key
        return

    def execute(self):
        self._originalValue = self._map[self._key]
        del self._map[self._key]
        return

    def unexecute(self):
        self._map[self._key] = self._originalValue
        return

    # END DeleteKeyCommand
    pass

class CommandBuilder1(CommandModule.CommandBuilder):
    
    
    def addCommandsPreExecute(self, commands):

        map = self._command._map
        key = self._command._key

        commands.insert(0, AddKeyCommand(map, key))

        return


    # END class CommandBuilder1
    pass


class CommandBuilder2(CommandModule.CommandBuilder):

    def __init__(self, command, value):
        CommandModule.CommandBuilder.__init__(self, command)
        self._value = value
        return

    def addCommandsPostExecute(self, commands):

        map = self._command._map
        key = self._command._key

        commands.append(SetValueCommand(map, key, self._value))
        
        return

    # END class CommandBuilder
    pass

class FailCommandBuilder1(CommandModule.CommandBuilder):
    def addCommandsPreExecute(self, commands):
        commands.insert(0, FailCommand())
        return

    # END class FailCommandBuilder1
    pass

class FailCommandBuilder2(CommandModule.CommandBuilder):
    def addCommandsPreExecute(self, commands):
        commands.append(FailCommand())
        return

    # END class FailCommandBuilder2
    pass

class FailCommandBuilder3(CommandModule.CommandBuilder):
    def addCommandsPostExecute(self, commands):
        commands.append(FailCommand())
        return

    # END class FailCommandBuilder3
    pass


class TestCommand(unittest.TestCase):


    def testValidator1(self):
        map = {}
        key = 1
        validator = HasKeyValidator(map, key)
        assert not validator.validate()

        notValidator = CommandModule.NotValidator(validator)
        assert notValidator.validate()
        return

    def testValidator2(self):
        key = 1
        map = {key:None}
        validator = HasKeyValidator(map, key)
        assert validator.validate()

        notValidator = CommandModule.NotValidator(validator)
        assert not notValidator.validate()
        return


    def testAddKeyCommand1(self):
        """
        verify that the basic command execute works
        """
        map = {}
        key = 1
        command = AddKeyCommand(map, key)
        validator = CommandModule.NotValidator(
            HasKeyValidator(map, key)
            )
        command.validator(validator)
        command.do()

        assert key in map

        command.undo()
        assert not key in map

        return

    def testAddKeyCommand2(self):
        """
        verify that the basic command execute works
        """
        key = 1
        map = {key:None}
        command = AddKeyCommand(map, key)
        validator = CommandModule.NotValidator(
            HasKeyValidator(map, key)
            )
        command.validator(validator)

        self.assertRaises(CommandModule.ValidationError, command.do)

        return

    def testSetValueCommand1(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}
        command = SetValueCommand(map, key, value)
        validator = HasKeyValidator(map, key)
        command.validator(validator)

        self.assertRaises(CommandModule.ValidationError, command.do)

        return

    def testSetValueCommand2(self):
        """
        """
        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}
        command = SetValueCommand(map, key, newValue)
        validator = HasKeyValidator(map, key)
        command.validator(validator)

        command.do()
        assert map[key] == newValue
        
        command.undo()
        assert map[key] == originalValue

        return

    def testCompoundCommand1(self):
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)

        addKeyCommand = AddKeyCommand(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)
        addKeyCommand.validator(notHasKeyValidator)

        command.addCommand(addKeyCommand)

        setValueCommand = SetValueCommand(map, key, value)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        command.do()
        assert map[key] == value

        command.undo()

        assert not key in map
        
        return

    def testCompoundCommand2(self):
        """
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(hasKeyValidator)
        command.addCommand(deleteKeyCommand)

        command.do()
        assert not key in map

        command.undo()
        assert map[key] == originalValue
        
        return

    def testCompoundCommand3(self):
        """
        test that if a subcommand fails because of a validator, 
        then everything is rolled back
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        failValidator = CommandModule.VALIDATOR_FAIL

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(failValidator)
        command.addCommand(deleteKeyCommand)

        try:
            command.do()
            assert False, 'expected command to fail'
        except CommandModule.ValidationError, e:
            assert map[key] == originalValue, 'map[%s] = %s' % (key, map[key])
            pass
        
        return

    def testCompoundCommand4(self):
        """
        verifies that if a subcommand fails during execution
        then all subcommands are rolled back
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(hasKeyValidator)
        command.addCommand(deleteKeyCommand)

        failCommand = FailCommand()
        command.addCommand(failCommand)


        try:
            command.do()
            assert False, 'expected command to fail'
        except CommandModule.ExecutionError, e:
            assert map[key] == originalValue, 'map[%s] = %s' % (key, map[key])
            pass

        return


    def testCommandBuilder1(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        setValueCommand = SetValueCommand(map, key, value)
        commandBuilder = CommandBuilder1(setValueCommand)

        command.addCommand(setValueCommand)
        command.addCommandBuilder(setValueCommand, commandBuilder)

        command.do()

        assert map[key] == value

        command.undo()

        assert not key in map

        return

    def testCommandBuilder2(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        addKeyCommand = AddKeyCommand(map, key)
        commandBuilder = CommandBuilder2(addKeyCommand, value)

        command.addCommand(addKeyCommand)
        command.addCommandBuilder(addKeyCommand, commandBuilder)

        command.do()

        assert map[key] == value

        command.undo()

        assert not key in map

        return


    def testFailCommandBuilder(self):
        """
        this will tests that the command is undone
        if a command added by the command builder fails
        regardless if its added before or after the execution
        """

        key = 1
        value = 'foo'
        map = {}


        for builderClass in [FailCommandBuilder1,
                             FailCommandBuilder2,
                             FailCommandBuilder3]:

            command = CommandModule.CompositeCommand()

            addKeyCommand = AddKeyCommand(map, key)
            commandBuilder = CommandBuilder2(addKeyCommand, value)

            command.addCommand(addKeyCommand)
            command.addCommandBuilder(addKeyCommand, commandBuilder)

            failCommandBuilder = builderClass(addKeyCommand)
            command.addCommandBuilder(addKeyCommand, failCommandBuilder)

            self.assertRaises(CommandModule.ExecutionError, command.do)

            assert len(map) is 0
            pass

        return


    # END class TestCommand
    pass



class TestCommandManager(unittest.TestCase):

    def setUp(self):
        self.commandManager = CommandModule.CommandManager()
        return
        
    def testAddKeyCommand1(self):
        """
        verify that the basic command execute works
        """
        map = {}
        key = 1
        command = AddKeyCommand(map, key)
        validator = CommandModule.NotValidator(
            HasKeyValidator(map, key)
            )
        command.validator(validator)

        self.commandManager.do(command)

        assert key in map

        self.commandManager.undo()

        assert not key in map

        return


    def testAddKeyCommand2(self):
        """
        verify that the basic command execute works
        """
        key = 1
        map = {key:None}
        command = AddKeyCommand(map, key)
        validator = CommandModule.NotValidator(
            HasKeyValidator(map, key)
            )
        command.validator(validator)

        self.assertTrue(key in map)
        self.commandManager.do(command)
        self.assertTrue(key in map)

        return

    def testSetValueCommand1(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}
        command = SetValueCommand(map, key, value)
        validator = HasKeyValidator(map, key)
        command.validator(validator)

        self.assertFalse(key in map)
        self.assertFalse(self.commandManager.do(command))
        self.assertTrue(self.commandManager.stackTrace() is not None)
        self.assertFalse(key in map)

        return

    def testSetValueCommand2(self):
        """
        """
        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}
        command = SetValueCommand(map, key, newValue)
        validator = HasKeyValidator(map, key)
        command.validator(validator)

        self.commandManager.do(command)
        assert map[key] == newValue
        
        self.commandManager.undo()
        assert map[key] == originalValue

        return

    def testCompoundCommand1(self):
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)

        addKeyCommand = AddKeyCommand(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)
        addKeyCommand.validator(notHasKeyValidator)

        command.addCommand(addKeyCommand)

        setValueCommand = SetValueCommand(map, key, value)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        self.commandManager.do(command)
        assert map[key] == value

        self.commandManager.undo()
        assert not key in map
        
        return

    def testCompoundCommand2(self):
        """
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(hasKeyValidator)
        command.addCommand(deleteKeyCommand)

        self.commandManager.do(command)
        assert not key in map

        self.commandManager.undo()
        assert map[key] == originalValue
        
        return

    def testCompoundCommand3(self):
        """
        test that if a subcommand fails because of a validator, 
        then everything is rolled back
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        failValidator = CommandModule.VALIDATOR_FAIL

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(failValidator)
        command.addCommand(deleteKeyCommand)

        self.assertEquals(map[key], originalValue)
        self.assertFalse(self.commandManager.do(command))
        self.assertTrue(self.commandManager.stackTrace() is not None)
        self.assertEquals(map[key], originalValue)

        return



    def testCompoundCommand4(self):
        """
        verifies that if a subcommand fails during execution
        then all subcommands are rolled back
        """

        key = 1
        originalValue = 'original'
        newValue = 'foo'
        map = {key:originalValue}

        command = CommandModule.CompositeCommand()

        hasKeyValidator = HasKeyValidator(map, key)
        notHasKeyValidator = CommandModule.NotValidator(hasKeyValidator)

        setValueCommand = SetValueCommand(map, key, newValue)
        setValueCommand.validator(hasKeyValidator)
        command.addCommand(setValueCommand)

        deleteKeyCommand = DeleteKeyCommand(map, key)
        deleteKeyCommand.validator(hasKeyValidator)
        command.addCommand(deleteKeyCommand)

        failCommand = FailCommand()
        command.addCommand(failCommand)

        self.assertEquals(map[key], originalValue)
        self.assertFalse(self.commandManager.do(command))
        self.assertTrue(self.commandManager.stackTrace() is not None)
        self.assertEquals(map[key], originalValue)

        return


    def testCommandBuilder1(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        setValueCommand = SetValueCommand(map, key, value)
        commandBuilder = CommandBuilder1(setValueCommand)

        command.addCommand(setValueCommand)
        command.addCommandBuilder(setValueCommand, commandBuilder)

        self.commandManager.do(command)
        assert map[key] == value

        self.commandManager.undo()
        assert not key in map

        return

    def testCommandBuilder2(self):
        """
        """
        key = 1
        value = 'foo'
        map = {}

        command = CommandModule.CompositeCommand()

        addKeyCommand = AddKeyCommand(map, key)
        commandBuilder = CommandBuilder2(addKeyCommand, value)

        command.addCommand(addKeyCommand)
        command.addCommandBuilder(addKeyCommand, commandBuilder)

        self.commandManager.do(command)
        assert map[key] == value

        self.commandManager.undo()
        assert not key in map

        return


    def testFailCommandBuilder(self):
        """
        this will tests that the command is undone
        if a command added by the command builder fails
        regardless if its added before or after the execution
        """

        key = 1
        value = 'foo'
        map = {}


        for builderClass in [FailCommandBuilder1,
                             FailCommandBuilder2,
                             FailCommandBuilder3]:

            command = CommandModule.CompositeCommand()

            addKeyCommand = AddKeyCommand(map, key)
            commandBuilder = CommandBuilder2(addKeyCommand, value)

            command.addCommand(addKeyCommand)
            command.addCommandBuilder(addKeyCommand, commandBuilder)

            failCommandBuilder = builderClass(addKeyCommand)
            command.addCommandBuilder(addKeyCommand, failCommandBuilder)

            self.assertTrue(len(map) is 0)
            self.assertFalse(self.commandManager.do(command))
            self.assertTrue(self.commandManager.stackTrace() is not None)
            self.assertTrue(len(map) is 0)

            pass

        return



    pass



def configLogging():
    """
    this will be called by all the main functions 
    to use the default setup for logging
    """
    # define a basic logger to write to file
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)-8s %(message)s',
                        datefmt='%a, %d %b %Y %H:%M:%S',
                        filename='/tmp/pypatterns.log',
                        filemode='w')

    # define a handler to write to stderr
    # console = logging.StreamHandler()
    # set the level of this to verbosity of severity 'warning'
    # console.setLevel(logging.WARNING)
    # set a format which is simpler for console use
    # formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
    # tell the handler to use this format
    # console.setFormatter(formatter)
    # add the handler to the root logger
    # logging.getLogger('').addHandler(console)

    # end def configureLogging
    pass





def main():

    configLogging()

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCase, 'test'))
    
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import currypy
import logging


def FUNCTION_IDENTITY(object=None):
    return object

FUNCTION_TRUE = currypy.Curry(FUNCTION_IDENTITY, object=True)
FUNCTION_FALSE = currypy.Curry(FUNCTION_IDENTITY, object=False)


def FUNCTION_IDENTITY_2(x, y):
    return x is y

def FUNCTION_EQUIVALENCE_2(x, y):
    return x == y

class Filter(object):

    def __init__(self):
        self._matched = set([])
        self._notMatched = set([])
        self._shouldCache = True
        return
    
    def __repr__(self):
        return self.name()

    def name(self, value=None):
        if value is not None:
            self._name = value
        if not hasattr(self, '_name'):
            self._name = '%s' % self.__class__
        return self._name


    def wrap(self, iterable, objectFunction=FUNCTION_IDENTITY):
        """
        this will wrap a filter around a generator/iterator 
        so that the new generator returns only values
        produced by the previous generator, but filtered
        by the filter provided
        aFunction also allows for specifying 
        if some attribute should be returned 
        instead of the object instead 
        """
        for x in iterable:
            if self.matches(x):
                theObject = objectFunction(x)
                yield theObject
                pass
            pass
        pass

    def hasMatched(self, object):
        if isinstance(object, list):
            object = tuple(object)
        if isinstance(object, dict):
            return False
        return object in self._matched
    
    def hasNotMatched(self, object):
        if isinstance(object, list):
            object = tuple(object)
        if isinstance(object, dict):
            return False
        return object in self._notMatched
    
    def addMatched(self, object):
        if isinstance(object, list):
            object = tuple(object)
        if isinstance(object, dict):
            return
        self._matched.add(object)
        return
    
    def addNotMatched(self, object):
        if isinstance(object, list):
            object = tuple(object)
        if isinstance(object, dict):
            return
        self._notMatched.add(object)
        return

    def resetCachedMatches(self):
        self._matched = set([])
        self._notMatched = set([])
        return

    
    def clearCacheofObject(self, object):
        self._matched.discard(object)
        self._notMatched.discard(object)
        pass
    
    
    def matches(self, object):
        raise NotImplementedError

    pass

class ObjectFilter(Filter):
    def __init__(self, 
                 valueFunction, 
                 objectFunction=None,
                 compareFunction=None):
        
        Filter.__init__(self)
        
        self.expected = valueFunction

        # default the actual value to object being passed in
        if objectFunction is None:
            objectFunction = FUNCTION_IDENTITY
        self._objectFunction = objectFunction

        # default comparator to identity comparison
        if compareFunction is None:
            compareFunction = FUNCTION_IDENTITY_2
        self._compareFunction = compareFunction

        pass

    def actual(self, object):
        """
        give an object, retrieves the actual value to be compared
        """
        return self._objectFunction(object)

    def matches(self, object):

        if self._shouldCache:
            if self.hasMatched(object):
                return True
            if self.hasNotMatched(object):
                return False
            pass
        
        try:
            actualValue = self.actual(object)
            expectedValue = self.expected()
            compareValue = self._compareFunction(actualValue, expectedValue)
            if compareValue:
                self.addMatched(object)
            else:
                self.addNotMatched(object)
            return compareValue
        except Exception, e:
            logging.error('something is wrong, filter got exception "%s", returning False' % e)
            self.addNotMatched(object)
            return False

    # END class ObjectFilter
    pass


def FUNCTION_TRUE_1(x):
    return True

def FUNCTION_FALSE_1(x):
    return False

TRUE_FILTER = ObjectFilter(valueFunction=FUNCTION_TRUE,
                           objectFunction=FUNCTION_TRUE_1)
TRUE_FILTER.name('TrueFilter')
TRUE_FILTER._shouldCache = False

FALSE_FILTER = ObjectFilter(valueFunction=FUNCTION_FALSE,
                            objectFunction=FUNCTION_TRUE_1)
FALSE_FILTER.name('FalseFilter')
FALSE_FILTER._shouldCache = False


class ObjectKeyMatchesFilter(ObjectFilter):

    @staticmethod
    def match(objectToMatch, filterObject):
        
        if not filterObject._hasKeyFunction(objectToMatch):
            return False
        
        try:
            keyToMatch = filterObject._keyFunction(objectToMatch)
            return filterObject._filter.matches(keyToMatch)
        except Exception, e:
            logging.error('could not retrieve key >> %s' % e)
            pass
        return False

    @staticmethod
    def defaultHasKeyFunction(x):
        return True
    
    def __init__(self, filter=None, keyFunction=None, hasKeyFunction=None):
        ObjectFilter.__init__(
            self,
            valueFunction = FUNCTION_TRUE,
            objectFunction = lambda x: ObjectKeyMatchesFilter.match(x, self)
        )
        self._filter = filter
        self._keyFunction = keyFunction
        if hasKeyFunction is None:
            hasKeyFunction = ObjectKeyMatchesFilter.defaultHasKeyFunction
        self._hasKeyFunction = hasKeyFunction
            
        return

    # END class ObjectKeyMatchesFilter
    pass


class MemberOfObjectKeyMatchesFilter(Filter):
    
    def __init__(self, accumulator=None, filter=None,
                 keyFunction=None, hasKeyFunction=None):
        
        Filter.__init__(self)

        self._accumulator = accumulator
        self._filter = filter
        self._keyFunction = keyFunction
        if hasKeyFunction is None:
            hasKeyFunction = ObjectKeyMatchesFilter.defaultHasKeyFunction
        self._hasKeyFunction = hasKeyFunction
        pass

    def accumulator(self):
        return self._accumulator
    
    
    def matches(self, object):

        if self._shouldCache:
            if self.hasMatched(object):
                return True
            if self.hasNotMatched(object):
                return False
            pass

        result = False
        if self._hasKeyFunction(object):
            
            try:
                filter = self._filter
                keyToMatch = self._keyFunction(object)
                accumulator = self.accumulator()
                accumulator.reset()
                for member in keyToMatch:
                    value = filter.matches(member)
                    accumulator.add(value)
                    if accumulator.isDone():
                        break
                    pass
                result = accumulator.value()
            except Exception, e:
                logging.error('could not retrieve key >> %s' % e)
                pass
            
            pass
        
        if result:
            self.addMatched(object)
        else:
            self.addNotMatched(object)
        return result
    
    
    # END MemberOfObjectKeyMatchesFilter
    pass


class IdentityFilter(ObjectFilter):
    def __init__(self, value):
        ObjectFilter.__init__(
            self,
            valueFunction = currypy.Curry(FUNCTION_IDENTITY,
                                              object=value),
            objectFunction = FUNCTION_IDENTITY)
        self.name('IdentityFilter(%s)' % str(value))
        return

    # END class IdentityFilter
    pass


class EquivalenceFilter(ObjectFilter):
    def __init__(self, value):
        ObjectFilter.__init__(
            self,
            valueFunction = currypy.Curry(FUNCTION_IDENTITY,
                                              object=value),
            objectFunction = FUNCTION_IDENTITY,
            #compareFunction = lambda x, y: x==y)
            compareFunction = FUNCTION_EQUIVALENCE_2
        )
        self.name('EquivalenceFilter(%s)' % value)
        return

    # END class EquivalenceFilter
    pass




class NameFilter(ObjectKeyMatchesFilter):
    @staticmethod
    def objectName(symbol):
        """
        given a symbol, return the
        string that represents the object's name
        """
        return symbol.name()
    
    def __init__(self, value):
        ObjectKeyMatchesFilter.__init__(
            self,
            EquivalenceFilter(value),
            NameFilter.objectName
        )
        self.name('NameFilter(%s)'%value)
        return

    # END class NameFilter
    pass


class IdFilter(ObjectKeyMatchesFilter):
    def __init__(self, value):
        ObjectKeyMatchesFilter.__init__(
            self,
            EquivalenceFilter(value),
            lambda x: x.id()
        )
        self.name('IdFilter(%s)'%value)
        return
    
    # END class IdFilter
    pass


class CompositeFilter(Filter):
    def __init__(self, accumulator):
        Filter.__init__(self)
        
        self.reset()
        self._accumulator = accumulator
        return


    def __repr__(self):
        return '%s:%s' % (self._accumulator, self.name())


    def reset(self):
        self._filters = []
        self.resetCachedMatches()
        pass
    
    def addFilter(self, filter):
        self._filters.append(filter)
        self.resetCachedMatches()
        return

    def removeFilter(self, filter):
        self._filters.remove(filter)
        self.resetCachedMatches()
        return

    
    def removeFilterUsingFilter(self, filter, recursive=False):
        theFilters = [x for x in self._filters]
        for theFilter in theFilters:
            if filter.matches(theFilter):
                #self._filters.remove(filter)
                self.removeFilter(filter)
                pass
            elif isinstance(theFilter, CompositeFilter) and recursive:
                theFilter.removeFilterUsingFilter(filter, recursive)
            pass
        return

    def accumulator(self):
        return self._accumulator

    def expected(self):
        expectedList = [filter.expected() for filter in self._filters]
        return "%s(%s)" % (self.accumulator(), expectedList)

    def actual(self, object):
        actualList = [filter.actual(object) for filter in self._filters]
        return "%s(%s)" % (self.accumulator(), actualList)

    def assertMatches(self, subfilter, object):
        return subfilter.matches(object)

    def matches(self, object):

        if self._shouldCache:
            if self.hasMatched(object):
                return True
            if self.hasNotMatched(object):
                return False
            pass
        
        accumulator = self.accumulator()
        accumulator.reset()
        for filter in self._filters:
            value = False
            try:
                value = self.assertMatches(filter, object)
            except Exception, e:
                pass
            accumulator.add(value)
            if accumulator.isDone():
                break
            pass

        result = accumulator.value()
        if result:
            self.addMatched(object)
        else:
            self.addNotMatched(object)
        return result
    

    # END class CompositeFilter
    pass

class Accumulator(object):
    def __init__(self):
        self.reset()
        return

    def reset(self):
        self._hasValue = False

    def add(self, value):
        raise NotImplementedError

    def isDone(self):
        return self._hasValue

    def value(self):
        return self._value

class AndOrAccumulator(Accumulator):
    """
    this accumulator will return true iff
    it is given [min,max] True values
    """
    def __init__(self, min=1, max=1):
        Accumulator.__init__(self)
        self._min = min
        self._max = max
        pass


    def reset(self):
        Accumulator.reset(self)
        self._count = 0
        self._value = False

    def add(self, value):
        if value is True:
            self._count = self._count + 1
        # has not gotten the necessary number of True
        if self._count < self._min:
            self._value = False
        # received more than max number of True
        elif self._count > self._max:
            self._value = False
            # value can no longer change anymore
            self._hasValue = True
        else:
            self._value = True
        return

class AndAccumulator(Accumulator):
    def __init__(self):
        Accumulator.__init__(self)
        return

    def __repr__(self):
        return "And"

    def reset(self):
        Accumulator.reset(self)
        self._value = True

    def add(self, value):
        self._value &= value
        if not self._value:
            self._hasValue = True

class OrAccumulator(Accumulator):
    def __init__(self):
        Accumulator.__init__(self)
        pass

    def __repr__(self):
        return "Or"

    def reset(self):
        Accumulator.reset(self)
        self._value = False

    def add(self, value):
        self._value |= value
        if self._value:
            self._hasValue = True

class NotAccumulator(Accumulator):
    def __init__(self):
        Accumulator.__init__(self)
        pass

    def __repr__(self):
        return "Not"

    def add(self, value):
        self._value = not value
        self._hasValue = True




def constructAndFilter():
    return CompositeFilter(AndAccumulator())

def constructOrFilter():
    return CompositeFilter(OrAccumulator())

def constructNotFilter():
    return CompositeFilter(NotAccumulator())




import unittest

import TestCommand
import TestFilter
import TestRelational
import TestRelationalCommand
import TestRelationalPickle

def additional_tests():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestFilter.TestCase, 'test'))
    suite.addTest(unittest.makeSuite(TestRelational.TestCase, 'test'))
    suite.addTest(unittest.makeSuite(TestRelationalCommand.TestCase, 'test'))
    suite.addTest(unittest.makeSuite(TestRelationalPickle.TestCase, 'test'))
    return suite




import pypatterns.command as CommandModule

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule


class CreateTableCommand(CommandModule.Command):

    def __init__(self, name, columns,
                 addToContextFunction, removeFromContextFunction):

        CommandModule.Command.__init__(self)

        self._addToContextFunction = addToContextFunction
        self._removeFromContextFunction = removeFromContextFunction

        self._tableName = name
        self._tableColumns = columns

        return


    def execute(self):

        table = RelationalModule.createTable(
            self._tableName, self._tableColumns)

        self._addToContextFunction(table)

        self._table = table
        return

    def unexecute(self):
        self._removeFromContextFunction(self._table)
        return

    pass


class AddRowCommand(CommandModule.Command):


    def __init__(self, table, addToContextFunction=None):
        CommandModule.Command.__init__(self)
        self._table = table
        self._addToContextFunction = addToContextFunction
        return

    def execute(self):
        row = self._table.addRow()
        self._row = row
        return

    def unexecute(self):
        filter = FilterModule.IdentityFilter(self._row)
        self._table.removeRows(filter)
        return
    
    # END class AddRowCommand
    pass


class SetColumnValueCommand(CommandModule.Command):

    def __init__(self, row, column, value):
        CommandModule.Command.__init__(self)
        self._row = row
        self._column = column
        self._newValue = value
        return

    def execute(self):
        self._originalValue = self._row.getColumn(self._column)
        self._row.setColumn(self._column, self._newValue)
        return

    def unexecute(self):
        self._row.setColumn(self._column, self._originalValue)
        return


    # END class SetColumnValueCommand
    pass

class SetColumnValueUsingRowFilterCommand(CommandModule.Command):

    def __init__(self, rowFilter, table, column, value):
        CommandModule.Command.__init__(self)
        self._rowFilter = rowFilter
        self._table = table
        self._column = column
        self._newValue = value
        return
    
    def execute(self):
        map = {}
        for row in self._table.retrieveForModification(self._rowFilter):
            map[row] = row.getColumn(self._column)
            row.setColumn(self._column, self._newValue)
            pass
        self._oldValueMap = map
        return

    def unexecute(self):
        for row, oldValue in self._oldValueMap.iteritems():
            row.setColumn(self._column, oldValue)
            pass

        return

    # END class SetColumnValueUsingRowFilterCommand
    pass


class SetColumnValueCommandBuilder(CommandModule.CommandBuilder):

    def __init__(self, command, column, value):
        CommandModule.CommandBuilder.__init__(self, command)
        self._column = column
        self._value = value
        return

    def addCommandsPostExecute(self, commands):
        
        commands.append(
            SetColumnValueCommand(
                self._command._row,
                self._column,
                self._value)
            )
        
        return

    # END class SetColumnValueCommandBuilder
    pass


class RemoveRowsCommand(CommandModule.Command):

    def __init__(self, table, filter):
        CommandModule.Command.__init__(self)
        self._table = table
        self._filter = filter
        return

    def execute(self):
        # save off the rows to be deleted
        removedRows = self._table.removeRows(self._filter)
        self._removedRows = removedRows
        return

    def unexecute(self):
        # restore the deleted rows
        map(self._table.addRow, self._removedRows)
        return

    # END class RemoveRowsCommand
    pass




import unittest

import pypatterns.filter as FilterModule


class TestCase(unittest.TestCase):

    def testOrAccumulator1(self):
        accumulator = FilterModule.OrAccumulator()
        
        self.assertFalse(accumulator.value())

        accumulator.add(False)
        self.assertFalse(accumulator.value())

        accumulator.add(True)
        self.assertTrue(accumulator.value())

        return


    def testOrAccumulator2(self):
        accumulator = FilterModule.OrAccumulator()
        
        self.assertFalse(accumulator.value())

        accumulator.add(True)
        self.assertTrue(accumulator.value())

        accumulator.add(False)
        self.assertTrue(accumulator.value())

        return


    # END class TestCase
    pass



import os
import sys
import unittest
import logging

APP_ROOT = os.getenv('APP_ROOT')

sys.path.insert(0, '%s/currypy/src' % APP_ROOT)
import currypy

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

sys.path.insert(0,"../data")

class TestCase(unittest.TestCase):

    COLUMNS = ['column1', 'column2', 'column3']
    
    
    def testTable(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test',columns)
        
        rowValuesList = [
            [1,2,3],
            [1,'2','3'],
            [None,None,[]]
        ]
        for rowValues in rowValuesList:
            row = table.addRow()
            map(row.setColumn, columns, rowValues)

        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1',
            FilterModule.ObjectFilter(
                valueFunction=lambda: 1,
                objectFunction=lambda x: x)
            )
        
        for actualValues, expectedValues in \
            zip(table.retrieve(columnValueFilter, columns[1:]), rowValuesList):
            
            assert actualValues == expectedValues[1:]
            
            
        return
    
    def testRow(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test',columns)
        
        expectedValues = [1,2,3]
        row1 = table.addRow()
        map(row1.setColumn, columns, expectedValues)
        for column, expectedValue in zip(columns, expectedValues):
            assert row1.getColumn(column) == expectedValue
            pass
        
        expectedValues = [RelationalModule.Table.NULL, 'foo', 'bar']
        row2 = table.addRow()
        map(row2.setColumn, columns[1:], expectedValues[1:])
        for column, expectedValue in zip(columns, expectedValues):
            assert row2.getColumn(column) == expectedValue
            pass
        
        return

    
    def testColumnValueFilter(self):
        table = self.buildDefaultTable()

        columns = TestCase.COLUMNS
        row = table.rows()[0]
        rowValues = [1, '1', None]

        for column, rowValue in zip(columns, rowValues):
            valueFilter = FilterModule.IdentityFilter(rowValue)
            columnValueFilter = RelationalModule.ColumnValueFilter(column, valueFilter)
            assert columnValueFilter.matches(row)

        return

    
    def testRemoveRows(self):
        table = self.buildDefaultTable()

        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1', FilterModule.IdentityFilter(1))
        
        table.removeRows(columnValueFilter)

        assert len(table.rows()) is 0
        
    
    def testModifyRow(self):
        table = self.buildDefaultTable()

        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1', FilterModule.IdentityFilter(1))
        for row in table.retrieveForModification(columnValueFilter):
            row.setColumn('column1', 2)
            
        columnValueFilter = RelationalModule.ColumnValueFilter(
            'column1', FilterModule.IdentityFilter(2))
        for rowValues in table.retrieve(columnValueFilter, ['column1']):
            assert rowValues[0] is 2
        
        return
    

    def buildDefaultTable(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test', columns)

        row = table.addRow()
        rowValues = [1, '1', None]
        map(row.setColumn, columns, rowValues)
        
        return table

    
    def testJoins(self):
        
        employeeTable = getEmployeeTable()
        departmentTable = getDepartmentTable()
        
        leftColumns = {'surname':'surname'}
        rightColumns = {
            'id':'department id',
            'name':'department name'
        }
        theFilter = RelationalModule.JoinFilter(
            {'department id':'id'}
        )
        theOperator = currypy.Curry(
            RelationalModule.defaultJoinOperator,
            leftColumnMap = leftColumns,
            rightColumnMap = rightColumns
        )
        joinInfo = {
            'left':employeeTable,
            'right':departmentTable,
            'filter':theFilter,
            'operator':theOperator,
            'left columns':leftColumns,
            'right columns':rightColumns,
            'name':'join',
        }
        
        
        for joinType, expectedResults in [
            (RelationalModule.Table.JOIN_INNER, getExpectedInnerJoinResults()),
            (RelationalModule.Table.JOIN_LEFT, getExpectedLeftJoinResults()),
            (RelationalModule.Table.JOIN_RIGHT, getExpectedRightJoinResults()),
            (RelationalModule.Table.JOIN_FULL, getExpectedFullJoinResults())]:
        
            joinInfo['join type']  = joinType
        
            actualResult = RelationalModule.join(joinInfo)

            assert expectedResults == actualResult
        return
    
    
    # END class TestCase
    pass


def getExpectedInnerJoinResults():
    expectedResults = RelationalModule.createTable(
        'join', ['surname', 'department id', 'department name'])
    for surname, id, name in [('Smith', 34, 'Clerical'),
                              ('Jones', 33, 'Engineering'),
                              ('Robinson', 34, 'Clerical'),
                              ('Steinberg', 33, 'Engineering'),
                              ('Rafferty', 31, 'Sales')]:
        row = expectedResults.addRow()
        row.setColumn('surname', surname)
        row.setColumn('department id', id)
        row.setColumn('department name', name)
        pass
    return expectedResults


def getExpectedLeftJoinResults():
    expectedResults = RelationalModule.createTable(
        'join', ['surname', 'department id', 'department name'])
    for surname, id, name in [('Smith', 34, 'Clerical'),
                              ('Jones', 33, 'Engineering'),
                              ('Robinson', 34, 'Clerical'),
                              ('Steinberg', 33, 'Engineering'),
                              ('Rafferty', 31, 'Sales'),
                              ('Jasper', RelationalModule.Table.NULL, RelationalModule.Table.NULL)]:
        row = expectedResults.addRow()
        row.setColumn('surname', surname)
        row.setColumn('department id', id)
        row.setColumn('department name', name)
        pass
    return expectedResults


def getExpectedRightJoinResults():
    expectedResults = RelationalModule.createTable(
        'join', ['surname', 'department id', 'department name'])
    for surname, id, name in [('Smith', 34, 'Clerical'),
                              ('Jones', 33, 'Engineering'),
                              ('Robinson', 34, 'Clerical'),
                              ('Steinberg', 33, 'Engineering'),
                              ('Rafferty', 31, 'Sales'),
                              (RelationalModule.Table.NULL, 35, 'Marketing')]:
        row = expectedResults.addRow()
        row.setColumn('surname', surname)
        row.setColumn('department id', id)
        row.setColumn('department name', name)
        pass
    return expectedResults


def getExpectedFullJoinResults():
    expectedResults = RelationalModule.createTable(
        'join', ['surname', 'department id', 'department name'])
    for surname, id, name in [('Smith', 34, 'Clerical'),
                              ('Jones', 33, 'Engineering'),
                              ('Robinson', 34, 'Clerical'),
                              ('Jasper', RelationalModule.Table.NULL, RelationalModule.Table.NULL),
                              ('Steinberg', 33, 'Engineering'),
                              ('Rafferty', 31, 'Sales'),
                              (RelationalModule.Table.NULL, 35, 'Marketing')]:
        row = expectedResults.addRow()
        row.setColumn('surname', surname)
        row.setColumn('department id', id)
        row.setColumn('department name', name)
        pass
    return expectedResults


def getEmployeeTable():
    
    columns = ['surname', 'department id']
    table = RelationalModule.createTable(
        'employees', 
        columns
    )
    for surname, deptid in [('Rafferty', 31),
                            ('Jones', 33),
                            ('Steinberg', 33),
                            ('Robinson', 34),
                            ('Smith', 34),
                            ('Jasper', None)]:
        row = table.addRow()
        if surname:
            row.setColumn('surname', surname)
        if deptid:
            row.setColumn('department id', deptid)
    return table


def getDepartmentTable():
    
    columns = ['id', 'name']
    table = RelationalModule.createTable(
        'department', columns)
    for id, name in [(31, 'Sales'),
                     (33, 'Engineering'),
                     (34, 'Clerical'),
                     (35, 'Marketing')]:
        row = table.addRow()
        row.setColumn('id', id)
        row.setColumn('name', name)
    return table
    

def main():
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




from __future__ import with_statement

import os
import pickle
import sys
import unittest
import logging

APP_ROOT = os.getenv('APP_ROOT')

import currypy

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

sys.path.insert(0,"../data")

class TestCase(unittest.TestCase):

    COLUMNS = ['column1', 'column2', 'column3']

    PICKLE_PATH = os.path.sep + os.path.join('tmp', 'TestRelationalPickle.pickle')

    def setUp(self):
        return
    
    def tearDown(self):
        if os.path.exists(TestCase.PICKLE_PATH):
            os.unlink(TestCase.PICKLE_PATH)
        return
    
    
    def testTable(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test',columns)

        rowValuesList = [
            [1,2,3],
            [1,'2','3'],
            [None,None,[]]
        ]
        for rowValues in rowValuesList:
            row = table.addRow()
            map(row.setColumn, columns, rowValues)

        self.assertPickleable(table)

        unpickledTable = self.assertJsonPickleable(table)
        self.assertEquals(table.rowCount(), unpickledTable.rowCount())
        for actualValues, expectedValues in zip(unpickledTable.retrieve(columns=['column1', 'column2', 'column3']), rowValuesList):
            self.assertEquals(actualValues, expectedValues)

        return
    
    
    def testRow(self):
        columns = TestCase.COLUMNS
        table = RelationalModule.createTable('test',columns)

        expectedValues = [1,2,3]
        row1 = table.addRow()
        map(row1.setColumn, columns, expectedValues)
        for column, expectedValue in zip(columns, expectedValues):
            assert row1.getColumn(column) == expectedValue
            pass

        self.assertPickleable(row1)

        unpickledRow = self.assertJsonPickleable(row1)
        self.assertEquals(row1.values(), unpickledRow.values())

        return



    def assertPickleable(self, objectToPickle):
        with open(TestCase.PICKLE_PATH, 'w') as f:
            pickle.dump(objectToPickle, f)

        with open(TestCase.PICKLE_PATH, 'r') as f:
            newObject = pickle.load(f)
        return


    def assertJsonPickleable(self, objectToPickle):

        import jsonpickle
        pickle = jsonpickle.encode(objectToPickle)
        
        unpickledObject = jsonpickle.decode(pickle)

        return unpickledObject


    # END class TestCase
    pass





def main():
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import os
import sys
import unittest
import logging

APP_ROOT = os.getenv('APP_ROOT')

sys.path.insert(0, '%s/currypy/src' % APP_ROOT)
sys.path.insert(0, '%s/pypatterns/src' % APP_ROOT)

import pypatterns.filter as FilterModule

class Resource(object):

    def name(self, value=None):
        if value is not None:
            self._name = value
        if not hasattr(self, '_name'):
            self._name = None
        return self._name
    
    # END class
    pass

class TestCase(unittest.TestCase):

    def testNotFilter(self):
        name = "test"

        # create the filter
        aFilter = FilterModule.constructNotFilter()
        aFilter.addFilter(FilterModule.NameFilter(name))

        aResource = Resource()
        aResource.name(name)
        assert aFilter.matches(aResource) is False, \
            "expected notfilter to not match resource named %s" % aResource.name()
        
        name = "wrong name"
        aResource = Resource()
        aResource.name(name)
        assert aFilter.matches(aResource), \
            "expected notfilter to match resource named %s" % aResource.name()

        pass

    def testAndOrFilter(self):
        name1 = "test1"
        name2 = "test2"

        resource1 = Resource()
        resource1.name(name1)
        resource2 = Resource()
        resource2.name(name2)

        # test to make sure that if min passes, succeeds
        aFilter = FilterModule.CompositeFilter(FilterModule.AndOrAccumulator())
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))
        assert aFilter.matches(resource1) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource1.name()
        assert aFilter.matches(resource2), \
            "expected andor(1,1) filter to accept resource %s" %resource2.name()

        # test to make sure that if max passes, succeeds
        aFilter = FilterModule.CompositeFilter(FilterModule.AndOrAccumulator(2,2))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))
        assert aFilter.matches(resource1), \
            "expected andor(1,1) filter to accept resource %s" % resource1.name()
        assert aFilter.matches(resource2) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource2.name()

        # test to make sure that if too many Filters passes, fails
        aFilter = FilterModule.CompositeFilter(FilterModule.AndOrAccumulator(0,0))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))
        assert aFilter.matches(resource1) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource1.name()
        assert aFilter.matches(resource2) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource2.name()        

        # test to make sure that if not enough Filters pass, fails
        aFilter = FilterModule.CompositeFilter(FilterModule.AndOrAccumulator(3,3))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))
        assert aFilter.matches(resource1) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource1.name()
        assert aFilter.matches(resource2) is False, \
            "expected andor(1,1) filter to not accept resource %s" % resource2.name()        

        pass

    def testOrFilter(self):
        name1 = "test1"
        name2 = "test2"
        name3 = "test3"

        aFilter = FilterModule.constructOrFilter()
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))

        # if any passes, succeed
        aResource = Resource()
        aResource.name(name1)
        assert aFilter.matches(aResource), \
            "expected orfilter to match resource named %s" % aResource.name()

        aResource = Resource()
        aResource.name(name2)
        assert aFilter.matches(aResource), \
            "expected orfilter to match resource named %s" % aResource.name()

        aResource = Resource()
        aResource.name(name3)
        assert not aFilter.matches(aResource), \
            "expected orfilter to match resource named %s" % aResource.name()

        # run again
        aResource = Resource()
        aResource.name(name1)
        assert aFilter.matches(aResource), \
            "expected notfilter to match resource named %s" % aResource.name()
        aResource = Resource()
        aResource.name(name2)
        assert aFilter.matches(aResource), \
            "expected notfilter to match resource named %s" % aResource.name()
        aResource = Resource()
        aResource.name(name3)
        assert not aFilter.matches(aResource), \
            "expected notfilter to match resource named %s" % aResource.name()
        

    def testAndFilter(self):
        name1 = "test1"
        name2 = "test2"

        aFilter = FilterModule.constructAndFilter()
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name1))

        # if all passes, succeed
        aResource = Resource()
        aResource.name(name1)
        assert aFilter.matches(aResource), \
            "expected andfilter to match resource named %s" % aResource.name()

        # if none passes, fail
        aResource = Resource()
        aResource.name(name2)
        assert aFilter.matches(aResource) is False, \
            "expected andfilter to not match resource named %s" % aResource.name()

        # if only some passes, fail
        aFilter = FilterModule.constructAndFilter()
        aFilter.addFilter(FilterModule.NameFilter(name1))
        aFilter.addFilter(FilterModule.NameFilter(name2))
        aResource = Resource()
        aResource.name(name1)
        assert aFilter.matches(aResource) is False, \
            "expected andfilter to not match resource named %s" % aResource.name()

        pass

    def testOrFilter(self):
        pass

    def testObjectFilter(self):
        pass

    def testNameFilter(self):

        name = "test"

        aFilter = FilterModule.NameFilter(name)

        aResource = Resource()
        aResource.name(name)
        assert aFilter.matches(aResource), \
            "expected filter to match resource named %s" % aResource.name()

        aResource = Resource()
        aResource.name("wrong name")
        assert aFilter.matches(aResource) is False, \
            "expected filter to not match resource named %s" % aResource.name()

        pass

    def testGeneratorFilter(self):
        """
        this test the ability to apply a filter to values
        produced by an iterator and/or generator
        """
        lists = []
        functions = []

        numbers = range(1,11)

        lists.append(numbers)
        functions.append(lambda x: x)

        lists.append([(0,x) for x in numbers])
        functions.append(lambda x: x[1])

        for list, function in zip(lists, functions):

            mod2Filter = FilterModule.ObjectFilter(lambda: 0,
                                             lambda x: function(x) % 2)
            mod2Filtered = mod2Filter.wrap(list)

            mod5Filter = FilterModule.ObjectFilter(lambda: 0,
                                             lambda x: function(x) % 5)
            
            filtered = mod5Filter.wrap(mod2Filtered, 
                                       objectFunction=function)
            
            cardinality = 0
            for number in filtered:
                if cardinality > 0:
                    assert False, \
                        "expected only one value in generator"
                assert number is 10, \
                    "expected value to be 10, got %s" % number
                cardinality = cardinality + 1
                pass
        pass


    def testMemberOfObjectKeyMatchesFilter1(self):
        
        filter = FilterModule.MemberOfObjectKeyMatchesFilter(
            filter = FilterModule.IdentityFilter(2),
            accumulator = FilterModule.AndAccumulator(),
            keyFunction = lambda x: x.values()         
        )
        filter._shouldCache = False
        
        class X(object):
            pass
        
        obj = X()
        setattr(obj, 'values', lambda: [0, 2, 2])
        assert not filter.matches(obj)
        
        setattr(obj, 'values', lambda: [2, 2, 2])
        assert filter.matches(obj)
        
        return
    
    def testMemberOfObjectKeyMatchesFilter2(self):
        
        filter = FilterModule.MemberOfObjectKeyMatchesFilter(
            filter = FilterModule.IdentityFilter(2),
            accumulator = FilterModule.OrAccumulator(),
            keyFunction = lambda x: x.values()         
        )
        filter._shouldCache = False
        
        class X(object):
            pass
        
        obj = X()
        setattr(obj, 'values', lambda: [0, 2, 1])
        assert filter.matches(obj)
        
        setattr(obj, 'values', lambda: [0, 1, 3])
        assert not filter.matches(obj)
        
        return
    
    
    pass

def main():
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




from setuptools import setup, find_packages

setup(
    name='pypatterns',
    version='0.1.4',
    packages=find_packages(
        'src',exclude=["*.test", "*.test.*", "test.*", "test"]),
    package_dir={'':'src'},
    test_suite="test",
    install_requires = [
        'currypy>=0.1.0',
        'Reaction>=0.2.2',
        'jsonpickle>=0.3.1'
        ],

    # metadata for upload to PyPI
    author = "michael j pan",
    author_email = "mikepan@gmail.com",
    description = "generic patterns",
    license = "New BSD",
    keywords = "object relational command filter",
    url = "http://code.google.com/p/pypatterns/",
    long_description = "This package implements some common patterns in Python",
    platforms = ["All"]

)
